"use strict";

exports.__esModule = true;
exports.usePanEvents = usePanEvents;

var React = _interopRequireWildcard(require("react"));

var _TouchHistoryMath = _interopRequireDefault(require("./TouchHistoryMath"));

var _useResponder = require("../use-responder");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * PAN RESPONDER
 *
 * `PanResponder` uses the Responder System to reconcile several touches into
 * a single gesture. It makes single-touch gestures resilient to extra touches,
 * and can be used to recognize simple multi-touch gestures. For each handler,
 * it provides a `gestureState` object alongside the ResponderEvent object.
 *
 * By default, `PanResponder` holds an `InteractionManager` handle to block
 * long-running JS events from interrupting active gestures.
 *
 * A graphical explanation of the touch data flow:
 *
 * +----------------------------+             +--------------------------------+
 * | ResponderTouchHistoryStore |             |TouchHistoryMath                |
 * +----------------------------+             +----------+---------------------+
 * |Global store of touchHistory|             |Allocation-less math util       |
 * |including activeness, start |             |on touch history (centroids     |
 * |position, prev/cur position.|             |and multitouch movement etc)    |
 * |                            |             |                                |
 * +----^-----------------------+             +----^---------------------------+
 *      |                                          |
 *      | (records relevant history                |
 *      |  of touches relevant for                 |
 *      |  implementing higher level               |
 *      |  gestures)                               |
 *      |                                          |
 * +----+-----------------------+             +----|---------------------------+
 * | ResponderEventPlugin       |             |    |   Your App/Component      |
 * +----------------------------+             +----|---------------------------+
 * |Negotiates which view gets  | Low level   |    |             High level    |
 * |onResponderMove events.     | events w/   |  +-+-------+     events w/     |
 * |Also records history into   | touchHistory|  |   Pan   |     multitouch +  |
 * |ResponderTouchHistoryStore. +---------------->Responder+-----> accumulative|
 * +----------------------------+ attached to |  |         |     distance and  |
 *                                 each event |  +---------+     velocity.     |
 *                                            |                                |
 *                                            |                                |
 *                                            +--------------------------------+
 */
var currentCentroidX = _TouchHistoryMath.default.currentCentroidX,
    currentCentroidY = _TouchHistoryMath.default.currentCentroidY,
    currentCentroidXOfTouchesChangedAfter = _TouchHistoryMath.default.currentCentroidXOfTouchesChangedAfter,
    currentCentroidYOfTouchesChangedAfter = _TouchHistoryMath.default.currentCentroidYOfTouchesChangedAfter,
    previousCentroidXOfTouchesChangedAfter = _TouchHistoryMath.default.previousCentroidXOfTouchesChangedAfter,
    previousCentroidYOfTouchesChangedAfter = _TouchHistoryMath.default.previousCentroidYOfTouchesChangedAfter;
/**
 * Reset the gesture state to its initial values
 */

function initializeGestureState(gestureState) {
  gestureState.x = 0;
  gestureState.y = 0;
  gestureState.initialX = 0;
  gestureState.initialY = 0;
  gestureState.deltaX = 0;
  gestureState.deltaY = 0;
  gestureState.velocityX = 0;
  gestureState.velocityY = 0;
  gestureState.numberActiveTouches = 0; // All `gestureState` accounts for timeStamps up until:

  gestureState._accountsForMovesUpTo = 0;
}
/**
 * Take all recently moved touches, calculate how the centroid has changed just for those
 * recently moved touches, and append that change to an accumulator. This is
 * to (at least) handle the case where the user is moving three fingers, and
 * then one of the fingers stops but the other two continue.
 *
 * This is very different than taking all of the recently moved touches and
 * storing their centroid as `dx/dy`. For correctness, we must *accumulate
 * changes* in the centroid of recently moved touches.
 *
 * There is also some nuance with how we handle multiple moved touches in a
 * single event. Multiple touches generate two 'move' events, each of
 * them triggering `onResponderMove`. But with the way `PanResponder` works,
 * all of the gesture inference is performed on the first dispatch, since it
 * looks at all of the touches. Therefore, `PanResponder` does not call
 * `onResponderMove` passed the first dispatch. This diverges from the
 * typical responder callback pattern (without using `PanResponder`), but
 * avoids more dispatches than necessary.
 *
 * When moving two touches in opposite directions, the cumulative
 * distance is zero in each dimension. When two touches move in parallel five
 * pixels in the same direction, the cumulative distance is five, not ten. If
 * two touches start, one moves five in a direction, then stops and the other
 * touch moves fives in the same direction, the cumulative distance is ten.
 *
 * This logic requires a kind of processing of time "clusters" of touch events
 * so that two touch moves that essentially occur in parallel but move every
 * other frame respectively, are considered part of the same movement.
 *
 * x/y: If a move event has been observed, `(x, y)` is the centroid of the most
 * recently moved "cluster" of active touches.
 * deltaX/deltaY: Cumulative touch distance. Accounts for touch moves that are
 * clustered together in time, moving the same direction. Only valid when
 * currently responder (otherwise, it only represents the drag distance below
 * the threshold).
 */


function updateGestureStateOnMove(gestureState, touchHistory) {
  var movedAfter = gestureState._accountsForMovesUpTo; // Previous values

  var prevX = previousCentroidXOfTouchesChangedAfter(touchHistory, movedAfter);
  var prevY = previousCentroidYOfTouchesChangedAfter(touchHistory, movedAfter);
  var prevDeltaX = gestureState.deltaX;
  var prevDeltaY = gestureState.deltaY; // Next values

  var x = currentCentroidXOfTouchesChangedAfter(touchHistory, movedAfter);
  var y = currentCentroidYOfTouchesChangedAfter(touchHistory, movedAfter); //console.log(x, prevX, prevDeltaX)

  var deltaX = prevDeltaX + (x - prevX);
  var deltaY = prevDeltaY + (y - prevY); // TODO: This must be filtered intelligently.

  var dt = touchHistory.mostRecentTimeStamp - movedAfter;
  gestureState.deltaX = deltaX;
  gestureState.deltaY = deltaY;
  gestureState.numberActiveTouches = touchHistory.numberActiveTouches;
  gestureState.velocityX = (deltaX - prevDeltaX) / dt;
  gestureState.velocityY = (deltaY - prevDeltaY) / dt;
  gestureState.x = x;
  gestureState.y = y;
  gestureState._accountsForMovesUpTo = touchHistory.mostRecentTimeStamp;
}

function usePanEvents(targetRef, config) {
  var onStartShouldSetPan = config.onStartShouldSetPan,
      onStartShouldSetPanCapture = config.onStartShouldSetPanCapture,
      onMoveShouldSetPan = config.onMoveShouldSetPan,
      onMoveShouldSetPanCapture = config.onMoveShouldSetPanCapture,
      onPanGrant = config.onPanGrant,
      onPanStart = config.onPanStart,
      onPanMove = config.onPanMove,
      onPanEnd = config.onPanEnd,
      onPanRelease = config.onPanRelease,
      onPanReject = config.onPanReject,
      onPanTerminate = config.onPanTerminate,
      onPanTerminationRequest = config.onPanTerminationRequest,
      touchAction = config.touchAction;
  var gestureStateRef = React.useRef({
    // Useful for debugging
    gestureID: Math.random(),
    x: 0,
    y: 0,
    initialX: 0,
    initialY: 0,
    deltaX: 0,
    deltaY: 0,
    velocityX: 0,
    velocityY: 0,
    numberActiveTouches: 0,
    _accountsForMovesUpTo: 0
  });
  var responderConfig = {
    onStartShouldSetResponder: function onStartShouldSetResponder(event) {
      var gestureState = gestureStateRef.current;
      return onStartShouldSetPan != null ? onStartShouldSetPan(event, gestureState) : false;
    },
    onMoveShouldSetResponder: function onMoveShouldSetResponder(event) {
      var gestureState = gestureStateRef.current;
      return onMoveShouldSetPan != null ? onMoveShouldSetPan(event, gestureState) : false;
    },
    onStartShouldSetResponderCapture: function onStartShouldSetResponderCapture(event) {
      var gestureState = gestureStateRef.current;
      var touchHistory = event.touchHistory; // TODO: Actually, we should reinitialize the state any time
      // touches.length increases from 0 active to > 0 active.

      if (event.nativeEvent.touches.length === 1) {
        initializeGestureState(gestureState);
      }

      gestureState.numberActiveTouches = touchHistory.numberActiveTouches;
      return onStartShouldSetPanCapture != null ? onStartShouldSetPanCapture(event, gestureState) : false;
    },
    onMoveShouldSetResponderCapture: function onMoveShouldSetResponderCapture(event) {
      var gestureState = gestureStateRef.current;
      var touchHistory = event.touchHistory;
      updateGestureStateOnMove(gestureState, touchHistory);
      return onMoveShouldSetPanCapture != null ? onMoveShouldSetPanCapture(event, gestureState) : false;
    },
    onResponderGrant: function onResponderGrant(event) {
      var gestureState = gestureStateRef.current;
      gestureState.initialX = currentCentroidX(event.touchHistory);
      gestureState.initialY = currentCentroidY(event.touchHistory);
      gestureState.deltaX = 0;
      gestureState.deltaY = 0;

      if (onPanGrant != null) {
        onPanGrant(event, gestureState);
      }
    },
    onResponderReject: function onResponderReject(event) {
      if (onPanReject != null) {
        var _gestureState = gestureStateRef.current;
        onPanReject(event, _gestureState);
      }
    },
    onResponderStart: function onResponderStart(event) {
      var gestureState = gestureStateRef.current;
      var numberActiveTouches = event.touchHistory.numberActiveTouches;
      gestureState.numberActiveTouches = numberActiveTouches;

      if (onPanStart != null) {
        onPanStart(event, gestureState);
      }
    },
    onResponderMove: function onResponderMove(event) {
      var gestureState = gestureStateRef.current;
      var touchHistory = event.touchHistory; // Guard against the dispatch of two touch moves when there are two
      // simultaneously changed touches.

      if (gestureState._accountsForMovesUpTo === touchHistory.mostRecentTimeStamp) {
        return;
      } // Filter out any touch moves past the first one - we would have
      // already processed multi-touch geometry during the first event.


      updateGestureStateOnMove(gestureState, touchHistory);

      if (onPanMove != null) {
        onPanMove(event, gestureState);
      }
    },
    onResponderEnd: function onResponderEnd(event) {
      var gestureState = gestureStateRef.current;
      var numberActiveTouches = event.touchHistory.numberActiveTouches;
      gestureState.numberActiveTouches = numberActiveTouches;

      if (onPanEnd != null) {
        onPanEnd(event, gestureState);
      }
    },
    onResponderRelease: function onResponderRelease(event) {
      var gestureState = gestureStateRef.current;

      if (onPanRelease != null) {
        onPanRelease(event, gestureState);
      }

      initializeGestureState(gestureState);
    },
    onResponderTerminate: function onResponderTerminate(event) {
      var gestureState = gestureStateRef.current;

      if (onPanTerminate != null) {
        onPanTerminate(event, gestureState);
      }

      initializeGestureState(gestureState);
    },
    onResponderTerminationRequest: function onResponderTerminationRequest(event) {
      var gestureState = gestureStateRef.current;
      return onPanTerminationRequest != null ? onPanTerminationRequest(event, gestureState) : true;
    },
    touchAction: touchAction
  };
  React.useDebugValue(config);
  (0, _useResponder.useResponderEvents)(targetRef, responderConfig);
}